#include <iostream>
#include <string>
#include <cstddef>
#include <vector>
#include <sstream>
#include <iterator>
#include <fstream>
#include <stdexcept>

struct V3f
{
	V3f( float x = 0.f, float y = 0.f, float z = 0.f ) :
		m_x( x ),
		m_y( y ),
		m_z( z )
	{}

	float m_x, m_y, m_z;

	friend std::istream & operator >> ( std::istream & in, V3f & vec )
	{
		float x, y, z;

		if( in >> x >> y >> z )
		{
			vec.m_x = x;
			vec.m_y = y;
			vec.m_z = z;
		}
		
		return in;
	}

	friend std::ostream & operator << ( std::ostream & out, V3f const & vec )
	{
		return out << vec.m_x << " " << vec.m_y << " " << vec.m_z;
	}
};

typedef std::vector< V3f > VecV3f;

void loadV3fFromFile( VecV3f & vecs, std::string const & fileName )
{
	using std::string;
	using std::stringstream;
	using std::istream_iterator;

	std::ifstream file( fileName );

	if( !file )
	{
		throw std::runtime_error( "Unable to open file." );
	}

	string line;
	size_t index;
	stringstream ss( std::ios_base::in );

	size_t lineNum = 0;
	vecs.clear();

	while( getline( file, line ) )
	{	
		++lineNum;
		
		// remove comments	
		index = line.find_first_of( "#" );
		if( index != string::npos )
		{
			line.erase( index, string::npos );
		}

		// removes leading/tailing junk
		line.erase( 0, line.find_first_not_of( " \t\r\n\v\f" ) );
		index = line.find_last_not_of( " \t\r\n\v\f" )+1;
		if( index != string::npos )
		{
			line.erase( index, string::npos );
		}
		
		if( line.empty() )
		{
			continue; // empty or commented out line
		}
		
		ss.str( line );
		ss.clear();

		V3f v;
		ss >> v;

		if( !ss || !ss.eof() || ss.good() )
		{
			throw std::runtime_error( "Error read file: "
						 + line
						 + " (line: "
						 + std::to_string(lineNum)
						 +  ")" );
		}	
		else
		{
			vecs.push_back( v );
		}
	}
	file.close();
}		

int main()
{ 
	std::string file( "./test.txt" );

	VecV3f vecs;
	loadV3fFromFile( vecs, file );

	for( auto & v : vecs )
	{
		std::cout << v << std::endl;
	}
}
